import { marked } from "marked";
import { ref, computed, isRef, watchEffect, createApp } from "vue";
function usePreview(md) {
  const container = ref();
  const mdCompute = computed(() => isRef(md) ? md.value : md);
  watchEffect((onInvalidate) => {
    if (mdCompute.value && container.value) {
      try {
        if (typeof mdCompute.value === "string") {
          const html = marked.parse(mdCompute.value);
          if (html)
            container.value.innerHTML = html;
        } else {
          const app = createApp(mdCompute.value);
          app.mount(container.value);
        }
      } catch (error) {
        console.log(error);
      }
    }
    onInvalidate(() => {
      if (container.value)
        container.value.innerHTML = "";
    });
  });
  return {
    container
  };
}
export {
  usePreview as default
};
