import { ref, computed, isRef, watchEffect } from "vue";
import debounce from "lodash/debounce";
const useDebouncePlugin = (fetchInstance, { debounceWait, debounceLeading, debounceTrailing, debounceMaxWait }) => {
  const debouncedRef = ref();
  const options = computed(() => {
    const ret = {};
    const debounceLeading_ = isRef(debounceLeading) ? debounceLeading.value : debounceLeading;
    const debounceTrailing_ = isRef(debounceTrailing) ? debounceTrailing.value : debounceTrailing;
    const debounceMaxWait_ = isRef(debounceMaxWait) ? debounceMaxWait.value : debounceMaxWait;
    if (debounceLeading_ !== void 0) {
      ret.leading = debounceLeading_;
    }
    if (debounceTrailing_ !== void 0) {
      ret.trailing = debounceTrailing_;
    }
    if (debounceMaxWait_ !== void 0) {
      ret.maxWait = debounceMaxWait_;
    }
    return ret;
  });
  watchEffect((onInvalidate) => {
    if (debounceWait) {
      const _originRunAsync = fetchInstance.runAsync.bind(fetchInstance);
      debouncedRef.value = debounce(
        (callback) => {
          callback();
        },
        debounceWait,
        options.value
      );
      fetchInstance.runAsync = (...args) => {
        return new Promise((resolve, reject) => {
          var _a;
          (_a = debouncedRef.value) == null ? void 0 : _a.call(debouncedRef, () => {
            _originRunAsync(...args).then(resolve).catch(reject);
          });
        });
      };
      onInvalidate(() => {
        var _a;
        (_a = debouncedRef.value) == null ? void 0 : _a.cancel();
        fetchInstance.runAsync = _originRunAsync;
      });
    }
  });
  if (!debounceWait) {
    return {};
  }
  return {
    onCancel: () => {
      var _a;
      (_a = debouncedRef.value) == null ? void 0 : _a.cancel();
    }
  };
};
export {
  useDebouncePlugin as default
};
