import { ref, isRef } from "vue";
const useLoadingDelayPlugin = (fetchInstance, { loadingDelay }) => {
  const timerRef = ref();
  if (isRef(loadingDelay) ? !loadingDelay.value : !loadingDelay) {
    return {};
  }
  const cancelTimeout = () => {
    if (timerRef.value) {
      clearTimeout(timerRef.value);
    }
  };
  return {
    onBefore: () => {
      cancelTimeout();
      timerRef.value = setTimeout(() => {
        fetchInstance.setState({
          loading: true
        });
      }, isRef(loadingDelay) ? loadingDelay.value : loadingDelay);
      return {
        loading: false
      };
    },
    onFinally: () => {
      cancelTimeout();
    },
    onCancel: () => {
      cancelTimeout();
    }
  };
};
export {
  useLoadingDelayPlugin as default
};
