import { ref, watchEffect, isRef } from "vue";
import isDocumentVisible from "../utils/isDocumentVisible";
import subscribeReVisible from "../utils/subscribeReVisible";
const usePollingPlugin = (fetchInstance, { pollingInterval, pollingWhenHidden = true }) => {
  const timerRef = ref();
  const unsubscribeRef = ref();
  const stopPolling = () => {
    var _a;
    if (timerRef.value) {
      clearInterval(timerRef.value);
    }
    (_a = unsubscribeRef.value) == null ? void 0 : _a.call(unsubscribeRef);
  };
  watchEffect(() => {
    if (isRef(pollingInterval) ? !(pollingInterval == null ? void 0 : pollingInterval.value) : !pollingInterval) {
      stopPolling();
    }
  });
  if (isRef(pollingInterval) ? !(pollingInterval == null ? void 0 : pollingInterval.value) : !pollingInterval) {
    return {};
  }
  return {
    onBefore: () => {
      stopPolling();
    },
    onFinally: () => {
      if (!pollingWhenHidden && !isDocumentVisible()) {
        unsubscribeRef.value = subscribeReVisible(() => {
          fetchInstance.refresh();
        });
        return;
      }
      timerRef.value = setInterval(() => {
        fetchInstance.refresh();
      }, isRef(pollingInterval) ? pollingInterval == null ? void 0 : pollingInterval.value : pollingInterval);
    },
    onCancel: () => {
      stopPolling();
    }
  };
};
export {
  usePollingPlugin as default
};
