import { ref, watchEffect } from "vue";
import throttle from "lodash/throttle";
const useThrottlePlugin = (fetchInstance, { throttleWait, throttleLeading, throttleTrailing }) => {
  const throttledRef = ref();
  const options = {};
  if ((throttleLeading == null ? void 0 : throttleLeading.value) !== void 0) {
    options.leading = throttleLeading.value;
  }
  if ((throttleTrailing == null ? void 0 : throttleTrailing.value) !== void 0) {
    options.trailing = throttleTrailing.value;
  }
  watchEffect((onInvalidate) => {
    if (throttleWait) {
      const _originRunAsync = fetchInstance.runAsync.bind(fetchInstance);
      throttledRef.value = throttle(
        (callback) => {
          callback();
        },
        throttleWait,
        options
      );
      fetchInstance.runAsync = (...args) => {
        return new Promise((resolve, reject) => {
          var _a;
          (_a = throttledRef.value) == null ? void 0 : _a.call(throttledRef, () => {
            _originRunAsync(...args).then(resolve).catch(reject);
          });
        });
      };
      onInvalidate(() => {
        var _a;
        fetchInstance.runAsync = _originRunAsync;
        (_a = throttledRef.value) == null ? void 0 : _a.cancel();
      });
    }
  });
  if (!throttleWait) {
    return {};
  }
  return {
    onCancel: () => {
      var _a;
      (_a = throttledRef.value) == null ? void 0 : _a.cancel();
    }
  };
};
export {
  useThrottlePlugin as default
};
