import { ref, reactive, computed, onMounted, isRef, onUnmounted, toRefs } from "vue";
import Fetch from "./Fetch";
function useRequestImplement(service, options = {}, plugins = []) {
  const { manual = false, ready = true, ...rest } = options;
  const fetchOptions = {
    manual,
    ready,
    ...rest
  };
  const serviceRef = ref(service);
  const state = reactive({
    data: void 0,
    loading: false,
    params: void 0,
    error: void 0
  });
  const setState = (s, field) => {
    if (field) {
      state[field] = s;
    } else {
      state.data = s.data;
      state.loading = s.loading;
      state.error = s.error;
      state.params = s.params;
    }
  };
  const fetchInstance = computed(() => {
    const initState = plugins.map((p) => {
      var _a;
      return (_a = p == null ? void 0 : p.onInit) == null ? void 0 : _a.call(p, fetchOptions);
    }).filter(Boolean);
    return new Fetch(
      serviceRef,
      fetchOptions,
      setState,
      Object.assign({}, ...initState)
    );
  });
  fetchInstance.value.options = fetchOptions;
  fetchInstance.value.pluginImpls = plugins.map((p) => {
    return p(fetchInstance.value, fetchOptions);
  });
  onMounted(() => {
    if (!manual) {
      const params = fetchInstance.value.state.params || options.defaultParams || [];
      if (isRef(ready) ? ready.value : ready)
        fetchInstance.value.run(...params);
    }
  });
  onUnmounted(() => {
    fetchInstance.value.cancel();
  });
  return {
    ...toRefs(state),
    cancel: fetchInstance.value.cancel.bind(fetchInstance.value),
    refresh: fetchInstance.value.refresh.bind(fetchInstance.value),
    refreshAsync: fetchInstance.value.refreshAsync.bind(fetchInstance.value),
    run: fetchInstance.value.run.bind(fetchInstance.value),
    runAsync: fetchInstance.value.runAsync.bind(fetchInstance.value),
    mutate: fetchInstance.value.mutate.bind(fetchInstance.value)
  };
}
export {
  useRequestImplement as default
};
