import { ref, computed, watchEffect, watch } from "vue";
import { parse, stringify } from "query-string";
const baseParseConfig = {
  parseNumbers: false,
  parseBooleans: false
};
const baseStringifyConfig = {
  skipNull: false,
  skipEmptyString: false
};
const routerPushFn = (s) => location.hash = s;
const useUrlState = (initialState, options) => {
  const { parseOptions, stringifyOptions } = options || {};
  const mergedParseOptions = { ...baseParseConfig, ...parseOptions };
  const mergedStringifyOptions = {
    ...baseStringifyConfig,
    ...stringifyOptions
  };
  const initialStateRef = ref(
    typeof initialState === "function" ? initialState() : initialState || {}
  );
  const state = ref(initialStateRef.value);
  const [path] = location.hash.slice(1).split("?");
  const queryFromUrl = computed(() => {
    return parse(location.search, mergedParseOptions);
  });
  const targetQuery = computed(() => ({
    ...initialStateRef.value,
    ...queryFromUrl.value
  }));
  watchEffect(() => {
    state.value = {
      ...initialStateRef.value,
      ...queryFromUrl.value
    };
  });
  const setState = (s) => {
    const newQuery = typeof s === "function" ? s({
      ...targetQuery.value,
      ...state.value
    }) : s;
    state.value = {
      ...state.value,
      ...newQuery
    };
  };
  watch(state, (curr) => {
    routerPushFn(
      `${path}?${stringify(
        {
          ...initialStateRef.value,
          ...queryFromUrl.value,
          ...curr
        },
        mergedStringifyOptions
      )}`
    );
  });
  return { state, setState };
};
export {
  useUrlState as default
};
