"use strict";
const vue = require("vue");
const useLoadingDelayPlugin = (fetchInstance, { loadingDelay }) => {
  const timerRef = vue.ref();
  if (vue.isRef(loadingDelay) ? !loadingDelay.value : !loadingDelay) {
    return {};
  }
  const cancelTimeout = () => {
    if (timerRef.value) {
      clearTimeout(timerRef.value);
    }
  };
  return {
    onBefore: () => {
      cancelTimeout();
      timerRef.value = setTimeout(() => {
        fetchInstance.setState({
          loading: true
        });
      }, vue.isRef(loadingDelay) ? loadingDelay.value : loadingDelay);
      return {
        loading: false
      };
    },
    onFinally: () => {
      cancelTimeout();
    },
    onCancel: () => {
      cancelTimeout();
    }
  };
};
module.exports = useLoadingDelayPlugin;
