"use strict";
const vue = require("vue");
const isDocumentVisible = require("../utils/isDocumentVisible");
const subscribeReVisible = require("../utils/subscribeReVisible");
const _interopDefaultLegacy = (e) => e && typeof e === "object" && "default" in e ? e : { default: e };
const isDocumentVisible__default = /* @__PURE__ */ _interopDefaultLegacy(isDocumentVisible);
const subscribeReVisible__default = /* @__PURE__ */ _interopDefaultLegacy(subscribeReVisible);
const usePollingPlugin = (fetchInstance, { pollingInterval, pollingWhenHidden = true }) => {
  const timerRef = vue.ref();
  const unsubscribeRef = vue.ref();
  const stopPolling = () => {
    var _a;
    if (timerRef.value) {
      clearInterval(timerRef.value);
    }
    (_a = unsubscribeRef.value) == null ? void 0 : _a.call(unsubscribeRef);
  };
  vue.watchEffect(() => {
    if (vue.isRef(pollingInterval) ? !(pollingInterval == null ? void 0 : pollingInterval.value) : !pollingInterval) {
      stopPolling();
    }
  });
  if (vue.isRef(pollingInterval) ? !(pollingInterval == null ? void 0 : pollingInterval.value) : !pollingInterval) {
    return {};
  }
  return {
    onBefore: () => {
      stopPolling();
    },
    onFinally: () => {
      if (!pollingWhenHidden && !isDocumentVisible__default.default()) {
        unsubscribeRef.value = subscribeReVisible__default.default(() => {
          fetchInstance.refresh();
        });
        return;
      }
      timerRef.value = setInterval(() => {
        fetchInstance.refresh();
      }, vue.isRef(pollingInterval) ? pollingInterval == null ? void 0 : pollingInterval.value : pollingInterval);
    },
    onCancel: () => {
      stopPolling();
    }
  };
};
module.exports = usePollingPlugin;
