"use strict";
const vue = require("vue");
const queryString = require("query-string");
const baseParseConfig = {
  parseNumbers: false,
  parseBooleans: false
};
const baseStringifyConfig = {
  skipNull: false,
  skipEmptyString: false
};
const routerPushFn = (s) => location.hash = s;
const useUrlState = (initialState, options) => {
  const { parseOptions, stringifyOptions } = options || {};
  const mergedParseOptions = { ...baseParseConfig, ...parseOptions };
  const mergedStringifyOptions = {
    ...baseStringifyConfig,
    ...stringifyOptions
  };
  const initialStateRef = vue.ref(
    typeof initialState === "function" ? initialState() : initialState || {}
  );
  const state = vue.ref(initialStateRef.value);
  const [path] = location.hash.slice(1).split("?");
  const queryFromUrl = vue.computed(() => {
    return queryString.parse(location.search, mergedParseOptions);
  });
  const targetQuery = vue.computed(() => ({
    ...initialStateRef.value,
    ...queryFromUrl.value
  }));
  vue.watchEffect(() => {
    state.value = {
      ...initialStateRef.value,
      ...queryFromUrl.value
    };
  });
  const setState = (s) => {
    const newQuery = typeof s === "function" ? s({
      ...targetQuery.value,
      ...state.value
    }) : s;
    state.value = {
      ...state.value,
      ...newQuery
    };
  };
  vue.watch(state, (curr) => {
    routerPushFn(
      `${path}?${queryString.stringify(
        {
          ...initialStateRef.value,
          ...queryFromUrl.value,
          ...curr
        },
        mergedStringifyOptions
      )}`
    );
  });
  return { state, setState };
};
module.exports = useUrlState;
