import { ComputedRef } from 'vue';
import { default as Cookies_2 } from 'js-cookie';
import { createApp } from 'vue';
import type { DebouncedFunc } from 'lodash';
import type { ParseOptions } from 'query-string';
import { Ref } from 'vue';
import type { StringifyOptions } from 'query-string';
import { VueElement } from 'vue';
import { WatchSource } from 'vue';

declare interface Actions {
    setTrue: () => void;
    setFalse: () => void;
    set: (value: boolean) => void;
    toggle: () => void;
}

declare type Actions_2<K, T> = {
    set: (key: K, value: T) => void;
    get: (key: K) => T | undefined;
    remove: (key: K) => void;
    has: (key: K) => boolean;
    clear: () => void;
    setAll: (newMap: MapValue<K, T>) => void;
    reset: () => void;
};

declare interface Actions_3<T> {
    add: (value: T) => void;
    remove: (value: T) => void;
    has: (value: T) => boolean;
    clear: () => void;
    reset: () => void;
}

declare interface Actions_4<T> {
    setLeft: () => void;
    setRight: () => void;
    set: (value: T) => void;
    toggle: () => void;
}

declare type AsyncOrder = {
    task: ((resolve?: Resolve, reject?: interruptibleRejectType, index?: number) => void)[];
    option?: {
        delay?: number;
        onReady?: () => void;
        onSuccess?: (result: any) => void;
        onError?: (err: any) => void;
    };
};

declare type BasicTarget<T extends TargetType = Element> = (() => TargetValue<T>) | TargetValue<T> | Ref<TargetValue<T>>;

declare type BasicTarget_2<T extends TargetType_2 = Element> = (() => TargetValue_2<T>) | TargetValue_2<T> | Ref<TargetValue_2<T>>;

declare interface CachedData<TData = any, TParams = any> {
    data: TData;
    params: TParams;
    time: number;
}

declare type Data = {
    list: any[];
    [key: string]: any;
};

declare interface DebounceOptions {
    wait?: number;
    leading?: boolean;
    trailing?: boolean;
    maxWait?: number;
}

declare type DependencyList = WatchSource | any[] | any;

declare type DependencyList_2 = Ref[];

declare type Effect = (changes?: number[], previousDeps?: DependencyList_2, currentDeps?: DependencyList_2) => void | (() => void);

declare class EventEmitter<T> {
    private subscriptions;
    private emitEffectCache;
    constructor();
    useSubscription: (event: string, listener?: Subscription<T> | undefined) => void;
    emit: (event: string | number, ...args: T extends any[] ? any[] : any) => void;
    emitEffect: (event: string | number) => void;
    removeListener: (event: string) => void;
    clear: () => void;
}

declare const eventEmitterOverall: EventEmitter<unknown>;

declare type EventEmitterType<T = void> = EventEmitter<T> | typeof eventEmitterOverall;

declare class Fetch<TData, TParams extends any[] = any> {
    serviceRef: Ref<Service<TData, TParams>>;
    options: Options<TData, TParams, any>;
    setUpdataData: (s: any, key?: keyof FetchState<TData, TParams>) => void;
    initState: Partial<FetchState<TData, TParams>>;
    pluginImpls: PluginReturn<TData, TParams>[] | undefined;
    count: number;
    state: FetchState<TData, TParams>;
    constructor(serviceRef: Ref<Service<TData, TParams>>, options: Options<TData, TParams, any>, setUpdataData: (s: any, key?: keyof FetchState<TData, TParams>) => void, initState?: Partial<FetchState<TData, TParams>>);
    setState(s?: Partial<FetchState<TData, TParams>>): void;
    setData(data: any, key?: keyof FetchState<TData, TParams> | (keyof FetchState<TData, TParams>)[]): void;
    runPluginHandler(event: keyof PluginReturn<TData, TParams>, ...rest: any[]): any;
    runAsync(...params: TParams): Promise<TData>;
    run(...params: TParams): void;
    cancel(): void;
    refresh(): void;
    refreshAsync(): Promise<TData>;
    mutate(data?: TData | ((oldData?: TData) => TData | undefined)): void;
}

declare interface FetchState<TData, TParams extends any[]> {
    loading: boolean;
    params?: TParams;
    data?: TData;
    error?: Error | unknown;
}

declare interface IFuncUpdater<T> {
    (previousState?: T): T;
}

declare interface InfiniteScrollOptions<TData extends Data> {
    target?: BasicTarget<Element | Document>;
    isNoMore?: (data?: TData) => boolean;
    threshold?: number;
    manual?: boolean;
    reloadDeps?: DependencyList;
    onBefore?: () => void;
    onSuccess?: (data: TData) => void;
    onError?: (e: Error) => void;
    onFinally?: (data?: TData, e?: Error) => void;
}

declare type interruptibleRejectType = (error: any) => void;

declare type IProps = Record<string, any>;

declare type MapValue<K, T> = Iterable<readonly [K, T]>;

declare interface NetworkState {
    since?: Date;
    online?: boolean;
    rtt?: number;
    type?: string;
    downlink?: number;
    saveData?: boolean;
    downlinkMax?: number;
    effectiveType?: string;
}

declare type noop = (...args: any) => any;

declare type noop_2 = (...p: any) => void;

declare type noop_3 = (...args: any) => any;

declare type Options<TData, TParams extends any[], TPlugin> = {
    [K in keyof TPlugin]: TPlugin[K];
} & {
    manual?: boolean;
    onBefore?: (params: TParams) => void;
    onSuccess?: (data: TData, params: TParams) => void;
    onError?: (e: Error, params: TParams) => void;
    onFinally?: (params: TParams, data?: TData, e?: Error) => void;
    defaultParams?: TParams;
    refreshDeps?: WatchSource[] | any;
    refreshDepsAction?: () => void;
    loadingDelay?: number | Ref<number>;
    formatResult?: (data?: TData) => any;
    pollingInterval?: Ref<number> | number;
    pollingWhenHidden?: boolean;
    refreshOnWindowFocus?: Ref<boolean> | boolean;
    focusTimespan?: number;
    debounceWait?: number;
    debounceLeading?: Ref<boolean>;
    debounceTrailing?: Ref<boolean>;
    debounceMaxWait?: Ref<number>;
    throttleWait?: number;
    throttleLeading?: Ref<boolean>;
    throttleTrailing?: Ref<boolean>;
    cacheKey?: string;
    cacheTime?: number;
    staleTime?: number;
    setCache?: (data: CachedData<TData, TParams>) => void;
    getCache?: (params: TParams) => CachedData<TData, TParams> | undefined;
    retryCount?: number;
    retryInterval?: number;
    ready?: Ref<boolean> | boolean;
    [x: string]: any;
};

declare interface Options_10 {
    rootMargin?: string;
    threshold?: number | number[];
    root?: BasicTarget<Element>;
}

declare interface Options_11<T> {
    serializer?: (value: T) => string;
    deserializer?: (value: string) => T;
}

declare interface Options_12 {
    restoreOnUnmount?: boolean;
}

declare interface Options_13 {
    navigateMode?: "push" | "replace";
    parseOptions?: ParseOptions;
    stringifyOptions?: StringifyOptions;
}

declare interface Options_14<T> {
    wrapperTarget: Ref<TargetValue_3<HTMLElement>>;
    itemHeight: number | ((index: number, data: T) => number);
    overscan?: number;
}

declare interface Options_2 extends Cookies_2.CookieAttributes {
    defaultValue?: State | (() => State);
}

declare interface Options_3 {
    min?: number;
    max?: number;
}

declare interface Options_4 {
    draggable?: boolean;
    onDragStart?: (event: DragEvent) => void;
    onDragEnd?: (event: DragEvent) => void;
}

declare interface Options_5 {
    onFiles?: (files: File[], event?: DragEvent) => void;
    onUri?: (url: string, event?: DragEvent) => void;
    onDom?: (content: any, event?: DragEvent) => void;
    onText?: (text: string, event?: ClipboardEvent) => void;
    onDragEnter?: (event?: DragEvent) => void;
    onDragOver?: (event?: DragEvent) => void;
    onDragLeave?: (event?: DragEvent) => void;
    onDrop?: (event?: DragEvent) => void;
    onPaste?: (event?: ClipboardEvent) => void;
}

declare interface Options_6 {
    type?: 'js' | 'css';
    js?: Partial<HTMLScriptElement>;
    css?: Partial<HTMLStyleElement>;
}

declare type Options_7<T extends Target = Target> = {
    target?: T;
    capture?: boolean;
    once?: boolean;
    passive?: boolean;
};

declare interface Options_8 {
    onFocus?: (e: FocusEvent) => void;
    onBlur?: (e: FocusEvent) => void;
    onChange?: (isFocusWithin: boolean) => void;
}

declare interface Options_9 {
    onEnter?: () => void;
    onLeave?: () => void;
}

declare interface OptionsWithDefaultValue<T> extends Options_11<T> {
    defaultValue: T | IFuncUpdater<T>;
}

declare type Plugin_2<TData, TParams extends any[], TPlugin = any> = {
    (fetchInstance: Fetch<TData, TParams>, options: Options<TData, TParams, TPlugin>): PluginReturn<TData, TParams>;
    onInit?: (options: Options<TData, TParams, TPlugin>) => Partial<FetchState<TData, TParams>>;
};

declare interface PluginReturn<TData, TParams extends any[]> {
    onBefore?: (params: TParams) => ({
        stopNow?: boolean;
        returnNow?: boolean;
    } & Partial<FetchState<TData, TParams>>) | void;
    onRequest?: (service: Service<TData, TParams>, params: TParams) => {
        servicePromise?: Promise<TData>;
    };
    onSuccess?: (data: TData, params: TParams) => void;
    onError?: (e: Error, params: TParams) => void;
    onFinally?: (params: TParams, data?: TData, e?: Error) => void;
    onCancel?: () => void;
    onMutate?: (data: TData) => void;
}

declare type Position = {
    left: number;
    top: number;
};

declare type Resolve = (value: any) => void;

declare interface Result<TData, TParams extends any[]> {
    loading: Ref<boolean>;
    data?: Ref<TData>;
    error?: Ref<Error>;
    params: Ref<TParams | []>;
    cancel: Fetch<TData, TParams>['cancel'];
    refresh: Fetch<TData, TParams>['refresh'];
    refreshAsync: Fetch<TData, TParams>['refreshAsync'];
    run: Fetch<TData, TParams>['run'];
    runAsync: Fetch<TData, TParams>['runAsync'];
    mutate: Fetch<TData, TParams>['mutate'];
}

declare type ScrollListenController = (val: Position) => boolean;

declare type Service<TData, TParams extends any[]> = (...args: TParams) => Promise<TData>;

declare type Service_2<TData extends Data> = (currentData?: TData) => Promise<TData>;

declare type Size = {
    width: Ref<number>;
    height: Ref<number>;
};

declare type State = string | undefined;

declare type Status = 'unset' | 'loading' | 'ready' | 'error';

declare type StorageStateResultHasDefaultValue<T> = [
Ref<T> | Ref<undefined>,
(value?: T | IFuncUpdater<T> | undefined) => void
];

declare type Subscription<T> = ({ params, event }: SubscriptionParams<T>) => void;

declare type SubscriptionParams<T = any> = {
    params: T;
    event: string | number;
};

declare type Target = BasicTarget<HTMLElement | Element | Window | Document>;

declare type Target_2 = BasicTarget<Element | Document>;

declare type TargetType = HTMLElement | Element | Window | Document;

declare type TargetType_2 = HTMLElement | Element | Window | Document;

declare type TargetValue<T> = T | undefined | null;

declare type TargetValue_2<T> = T | undefined | null;

declare type TargetValue_3<T> = T | undefined | null;

declare interface ThrottleOptions {
    wait?: number;
    leading?: boolean;
    trailing?: boolean;
}

declare type UrlState = Record<string, any>;

export declare function useAsyncOrder({ task, option }: AsyncOrder): void;

export declare function useBoolean(defaultValue?: boolean): [Ref<boolean>, Actions];

export declare function useCookieState(cookieKey: string, options?: Options_2): readonly [Ref<State>, (newValue: State | ((prevState: State) => State), newOptions?: Cookies_2.CookieAttributes) => void];

export declare function useCounter(initialValue?: number, options?: Options_3): readonly [Ref<number>, {
    readonly inc: (delta?: number) => void;
    readonly dec: (delta?: number) => void;
    readonly set: (value: ValueParam) => void;
    readonly reset: () => void;
}];

export declare function useDarkMode(): [
ComputedRef<any>,
(value?: unknown) => void
];

export declare function useDebounce<T>(value: Ref<T>, options?: DebounceOptions): Ref<T>;

export declare function useDebounceFn<T extends noop>(fn: T, options?: DebounceOptions): {
    run: DebouncedFunc<T>;
};

export declare const useDrag: <T>(data: T, target: BasicTarget, options?: Options_4) => void;

export declare const useDrop: (target: BasicTarget, options?: Options_5) => void;

export declare function useEventEmitter<T = void>(options?: {
    global?: boolean;
}): EventEmitterType<T>;

export declare function useEventListener<K extends keyof HTMLElementEventMap>(eventName: K, handler: (ev: HTMLElementEventMap[K]) => void, options?: Options_7<HTMLElement>): void;

export declare function useEventListener<K extends keyof ElementEventMap>(eventName: K, handler: (ev: ElementEventMap[K]) => void, options?: Options_7<Element>): void;

export declare function useEventListener<K extends keyof DocumentEventMap>(eventName: K, handler: (ev: DocumentEventMap[K]) => void, options?: Options_7<Document>): void;

export declare function useEventListener<K extends keyof WindowEventMap>(eventName: K, handler: (ev: WindowEventMap[K]) => void, options?: Options_7<Window>): void;

export declare function useEventListener(eventName: string, handler: noop_2, options: Options_7): void;

export declare function useExternal(path?: string | Ref<string>, options?: Options_6): Ref<Status>;

export declare function useFavicon(href: string | Ref<string>): void;

export declare function useFocusWithin(target: BasicTarget, options?: Options_8): Ref<boolean>;

export declare function useHover(target: BasicTarget, options?: Options_9): Ref<boolean>;

export declare const useInfiniteScroll: <TData extends Data>(service: Service_2<TData>, options?: InfiniteScrollOptions<TData>) => {
    data: Ref<TData | undefined>;
    loading: ComputedRef<boolean>;
    loadingMore: Ref<boolean>;
    noMore: ComputedRef<boolean>;
    loadMore: () => void;
    loadMoreAsync: () => Promise<TData> | undefined;
    reload: () => void;
    reloadAsync: () => Promise<TData>;
    mutate: (mutateData: any) => void;
    scrollMethod: () => void;
    cancel: () => void;
};

export declare function useInterval(fn: () => void, delay: Ref<number | undefined> | number | undefined, options?: {
    immediate?: boolean;
}): void;

export declare function useInViewport(target: BasicTarget, options?: Options_10): readonly [Ref<boolean | undefined>, Ref<number | undefined>];

export declare const useLocalStorageState: <T>(key: string | Ref<string>, options?: OptionsWithDefaultValue<T> | undefined) => StorageStateResultHasDefaultValue<T>;

export declare function useLockFn<P extends any[] = any[], V extends any = any>(fn: (...args: P) => Promise<V>): (...args: P) => Promise<V | undefined>;

export declare function useMap<K, T>(initialValue?: MapValue<K, T>): [Ref<Map<K, T>>, Actions_2<K, T>];

export declare function useMedia(queries: any[], values: {
    [x: string]: any;
}, defaultValue: any): any;

export declare function useMouse(target?: BasicTarget): Ref<{
    screenX: number;
    screenY: number;
    clientX: number;
    clientY: number;
    pageX: number;
    pageY: number;
    elementX: number;
    elementY: number;
    elementH: number;
    elementW: number;
    elementPosX: number;
    elementPosY: number;
}>;

export declare function useNetwork(): NetworkState;

export declare function usePreview(md: Parameters<typeof createApp> | Ref<string> | string | VueElement): {
    container: Ref<Element | undefined>;
};

export declare function useRequest<TData, TParams extends any[], TPluginOptions, PluginsOptions extends any[] = Plugin_2<TData, TParams, TPluginOptions>[]>(service: Service<TData, TParams>, options?: Options<TData, TParams, PluginsOptions['length'] extends 1 ? TPluginOptions : PluginsOptions extends (infer P)[] ? P extends Plugin_2<TData, TParams, infer R> ? R : any : any>, plugins?: PluginsOptions): Result<TData, TParams>;

export declare function useScroll(target?: Target_2, shouldUpdate?: ScrollListenController): Ref<Position | undefined>;

export declare const useSessionStorageState: <T>(key: string | Ref<string>, options?: OptionsWithDefaultValue<T> | undefined) => StorageStateResultHasDefaultValue<T>;

export declare function useSet<T = any>(initialValue?: T[]): [state: Ref<Set<any>>, actions: Actions_3<T>];

/**
 *
 * @param {dom id节点或者 ref句柄} target
     */
 export declare function useSize(target: BasicTarget_2): Size | undefined;

 export declare function useThrottle<T>(value: Ref<T>, options?: ThrottleOptions): Ref<any>;

 export declare function useThrottleFn<T extends noop_3>(fn: T, options?: ThrottleOptions): {
     run: ComputedRef<DebouncedFunc<(...args: Parameters<T>) => ReturnType<T>>>;
     cancel: () => void;
     flush: () => ReturnType<T> | undefined;
 };

 export declare function useTimeout(fn: () => void, delay: Ref<number | undefined> | number | undefined, options?: {
     immediate?: boolean;
 }): void;

 export declare function useTitle(title: Ref<string>, options?: Options_12): void;

 export declare function useToggle<T = boolean>(): [Ref<T>, Actions_4<T>];

 export declare function useToggle<T = boolean>(defaultValue: T): [Ref<T>, Actions_4<T>];

 export declare function useToggle<T, U>(defaultValue: T, reverseValue: U): [Ref<T | U>, Actions_4<T | U>];

 export declare const useTrackedEffect: (effect: Effect, deps?: DependencyList_2 | undefined) => void;

 export declare function useUpdate(): {
     update: Ref<{}>;
     setUpdate: () => void;
 };

 export declare const useUrlState: <S extends UrlState = UrlState>(initialState?: S | (() => S) | undefined, options?: Options_13 | undefined) => {
     state: Ref<any>;
     setState: (s: Partial<{ [key in keyof S]: any; }> | ((prev: any) => Partial<{ [key in keyof S]: any; }>)) => void;
 };

 export declare const useVirtualList: <T = any>(list: Ref<T[]>, options: Options_14<T>) => readonly [Ref<{
     index: number;
     data: T;
 }[]>, {
     ref: (ele: any) => void;
     onScroll: (e: any) => void;
 }, (index: number) => void];

 export declare function useWhyDidYouUpdate(componentName: string, props: IProps): void;

 export declare function useWinResize(Action?: () => void): null;

 declare type ValueParam = number | ((c: number) => number);

 export { }
